import { globalLookup, globalSetting } from '@/hook/global';
import { Cell, ModuleView, Port } from '@/hook/render/yosys';
import { SkinManager } from '@/hook/skin';
import i18n from '@/i18n';
import { reactive } from 'vue';


const { t, locale } = i18n.global;
locale.value = globalSetting.language;

export const infoView = reactive({
    name: undefined,
    
    type: undefined,

    /**
     * @type {'port' | 'module'}
     */
    typeId: undefined,

    module: undefined,
    definition: undefined,
    width: undefined,
    direction: undefined,


    // module 渲染属性
    inputCount: undefined,
    outputCount: undefined,
    inoutCount: undefined,
    instanceCount: undefined,
    cellCount: undefined,
    instanceName: undefined,
    instanceDefinition: undefined,

    // edge 渲染属性
    from: undefined,
    to: undefined,

    /**
     * @description 显示一个 port 到当前信息窗口
     * @param {Port} port 
     */
    displayPort(port) {
        this.clear();
        this.name = port.name;
        this.module = port.view.name;
        this.type = t("common.port");
        this.typeId = 'port';

        this.definition = port.definition;
        this.width = port.width;
        this.direction = port.direction;
    },

    /**
     * @description 显示一个 module 到当前窗口
     * @param {ModuleView} module 
     * @param {Cell} [cell] 
     */
    displayModule(module, cell) {
        this.clear();

        const isTop = (cell === undefined);

        this.name = module.name;
        this.type = (!isTop) ? 
            t("common.instance"): 
            t('common.top-module');

        this.typeId = 'module';
        this.definition = module.definition;
        
        this.instanceName = (cell || {}).name;
        this.instanceDefinition = (cell || {}).definition;

        const portInfo = module.portInfo;
        this.inputCount = portInfo.input;
        this.outputCount = portInfo.output;
        this.inoutCount = portInfo.inout;

        const skinManager = globalLookup.skinManager;

        let instanceCount = 0;
        let cellCount = 0;

        this.instanceCount = 0;
        this.cellCount = 0;

        for (const cellname of module.nameToCell.keys()) {
            const cell = module.nameToCell.get(cellname);
            const skin = skinManager.querySkin(cell.type);
            
            if (skin) {
                cellCount ++;
            } else {
                instanceCount ++;
            }
        }

        this.instanceCount = instanceCount;
        this.cellCount = cellCount;
    },

    clear() {
        this.name = undefined;
        this.type = undefined;
        this.module = undefined;
        this.definition = undefined;
        this.direction = undefined;
        this.width = undefined;
        this.from = undefined;
        this.to = undefined;
        this.instanceName = undefined;
    }
});
